package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.support.enums.BlockType;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import com.mysql.cj.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description 音频处理器类
 * @Author lideyin
 * @Date 2020/5/15 16:29
 * @Version 1.0
 */
public class AudioTextHandler implements RichTextHandler{
    private static final Logger LOG = LoggerFactory.getLogger(AudioTextHandler.class);

    @Override
    public SectionCollectionBean handle(String richText) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[HTML-TEXT-HANDLER]--richText: " + richText);
        }

        if (StringUtils.isNullOrEmpty(richText)) {
            return new SectionCollectionBean();
        }

        SectionCollectionBean sectionCollection = new SectionCollectionBean();
        List<SectionBean> sectionList = new ArrayList<>();
        SectionBean section = new SectionBean();
        List<BlockBean> blockList = new ArrayList<>();

        if (validateAudioAnswer(richText)) {
            BlockBean blockBean = new BlockBean(BlockType.audio.name());
            blockBean.setValue(richText);
            blockList.add(blockBean);
        }

        section.setBlocks(blockList);
        sectionList.add(section);
        sectionCollection.setSections(sectionList);

        return sectionCollection;
    }

    /**
     * 校验音频答案格式是否正确
     *
     * @param studentAnswer
     * @return
     */
    private boolean validateAudioAnswer(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return false;
        }

        String regExp = "^(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(mp3)";
        return studentAnswer.matches(regExp);
    }
}
