package cn.com.qmth.examcloud.support.helper;

import cn.com.qmth.examcloud.commons.util.StringUtil;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.ExamPropertyCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.OrgPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.enums.FaceBiopsyScheme;
import cn.com.qmth.examcloud.support.privilege.PrivilegeDefine;
import cn.com.qmth.examcloud.support.privilege.PrivilegeManager;
import org.apache.commons.lang3.StringUtils;

/**
 * @Description 活体检测帮助类
 * @Author lideyin
 * @Date 2019/11/6 15:01
 * @Version 1.0
 */
public class FaceBiopsyHelper {
    /**
     * 获取活体检测方案
     *
     * @param rootOrgId
     * @return
     */
    public static FaceBiopsyScheme getFaceBiopsyScheme(Long rootOrgId) {
        OrgPropertyCacheBean orgProperty = CacheHelper.getOrgProperty(rootOrgId,
                Constants.IDENTIFICATION_OF_LIVING_BODY_SCHEME_KEY);
        if (orgProperty.getHasValue()) {
            if (FaceBiopsyScheme.FACE_ID.getCode().equals(orgProperty.getValue())) {
                return FaceBiopsyScheme.FACE_ID;
            }
            return FaceBiopsyScheme.NEW;
        }
        //默认使用旧活体检测方案(即faceId方案)
        return FaceBiopsyScheme.FACE_ID;
    }

    /**
     * 是否开启人脸
     *
     * @param rootOrgId
     * @param examId
     * @param studentId
     * @return
     */
    public static Boolean isFaceEnable(Long rootOrgId, Long examId, Long studentId) {
        //判断是否有人脸识别的权限
        String faceCheck = PrivilegeDefine.RootOrgFunctions.OnlineExamFunctions.FaceCheck.CODE;
        Boolean hasFaceCheckFunction = PrivilegeManager.judge(rootOrgId, faceCheck);

        //如果有权限，具体的人脸识别的配置值
        ExamPropertyCacheBean examConf = ExamCacheTransferHelper.getCachedExamProperty(examId,
                studentId, ExamProperties.IS_FACE_ENABLE.name());
        String examValue = examConf.getValue();

        if (!hasFaceCheckFunction) {
            return false;
        }

        if (StringUtils.isBlank(examValue)) {
            return false;
        }

        if (StringUtil.isTrue(examValue)) {
            return true;
        }

        return false;

    }

    /**
     * 是否开启人脸活体检测
     *
     * @param rootOrgId
     * @param examId
     * @param studentId
     * @return
     */
    public static Boolean isFaceVerify(Long rootOrgId, Long examId, Long studentId) {
        ExamSettingsCacheBean examSettings = CacheHelper.getExamSettings(examId);

        //如果未开启人脸识别，则直接返回false
        Boolean faceEnable = isFaceEnable(rootOrgId, examId, studentId);
        if (!faceEnable) {
            return false;
        }

        //人脸活体检测权限
        String IdentificationOfLivingBody = PrivilegeDefine.RootOrgFunctions.OnlineExamFunctions.IdentificationOfLivingBody.CODE;
        Boolean hasIdentificationOfLivingBodyFunction = PrivilegeManager
                .judge(examSettings.getRootOrgId(), IdentificationOfLivingBody);

        //人脸活体检测配置值
        ExamPropertyCacheBean examConf = CacheHelper.getExamProperty(examId, ExamProperties.IS_FACE_VERIFY.name());
        String examValue = examConf.getValue();

        if (!hasIdentificationOfLivingBodyFunction) {
            return false;
        }

        if (StringUtils.isBlank(examValue)) {
            return false;
        }

        if (StringUtil.isTrue(examValue)) {
            return true;
        }

        return false;
    }

    /**
     * 进入考试是否验证人脸识别(强制、非强制)
     *
     * @return
     */
    public static Boolean isFaceCheck(Long examId, Long studentId) {
        String isFaceCheck = ExamCacheTransferHelper.getCachedExamProperty(examId,
                studentId, ExamProperties.IS_FACE_CHECK.name()).getValue();
        if (StringUtils.isBlank(isFaceCheck)) {
            return false;
        }
        return Boolean.valueOf(isFaceCheck);
    }

    /**
     * 判断是否有人脸识别权限
     *
     * @param rootOrgId
     * @param examId
     * @return
     * @author WANGWEI
     */
    private Boolean faceCheckEnabled(Long rootOrgId, Long examId, Long studentId) {
        String faceCheck = PrivilegeDefine.RootOrgFunctions.OnlineExamFunctions.FaceCheck.CODE;
        Boolean hasFaceCheckFunction = PrivilegeManager.judge(rootOrgId, faceCheck);

        ExamPropertyCacheBean examConf = ExamCacheTransferHelper.getCachedExamProperty(examId,
                studentId, ExamProperties.IS_FACE_ENABLE.name());
        String examValue = examConf.getValue();

        if (!hasFaceCheckFunction) {
            return false;
        }

        if (StringUtils.isBlank(examValue)) {
            return false;
        }

        if (StringUtil.isTrue(examValue)) {
            return true;
        }

        return false;
    }
}
